# -*- coding: UTF-8 -*-
import os
from pefile import PE
from peutils import SignatureDatabase


__all__ = ["identify_packer", "open_sigs", "DB"]


__log = lambda l, m, lvl="debug": getattr(l, lvl)(m) if l else None
DB = os.path.join(os.path.dirname(__file__), "userdb.txt")


def identify_packer(pe, db=None, ep_only=True, logger=None):
    """ Identify the packer used in a given executable using the given signatures database.
    
    :param pe:      either the path to the executable file or a PE instance
    :param db:      path to the database
    :param ep_only: consider only entry point signatures
    :return:        return the matching packers
    """
    if not isinstance(pe, PE):
        path = pe
        pe = PE(pe)
        pe.path = path
    db = open_sigs(db, logger)
    if logger:
        logger.debug("Parsing PE file '%s'..." % getattr(pe, "path", "unknown path"))
    return db.match(pe, ep_only=ep_only) or []


def open_sigs(path, logger=None):
    """ Open a signatures database.
    
    :param path: path to the database
    :return:     SignatureDatabase instance
    """
    path = path or DB
    if logger:
        logger.debug("Opening signature database '%s'..." % path)
    with open(path, encoding="latin-1") as f:
        sigs = SignatureDatabase(data=f.read())
    return sigs




# -*- coding: UTF-8 -*-
import logging
import sys
from argparse import ArgumentParser, RawTextHelpFormatter
from os.path import exists
from time import perf_counter

from .__info__ import __author__, __copyright__, __email__, __license__, __source__, __version__
from .__init__ import identify_packer, DB


def valid_file(path):
    if not exists(path):
        raise ValueError("input file does not exist")
    return path


def main():
    """ Tool's main function """
    descr = "PEiD {}\n\nAuthor   : {} ({})\nCopyright: {}\nLicense  : {}\nSource   : {}\n" \
            "\nThis tool is an implementation in Python of the Packed Executable iDentifier (PEiD) in the scope of " \
            "packing detection for Windows PE files based on signatures.\n\n"
    descr = descr.format(__version__, __author__, __email__, __copyright__, __license__, __source__)
    examples = "usage examples:\n- " + "\n- ".join([
        "peid program.exe",
        "peid program.exe -b",
        "peid program.exe --db custom_sigs_db.txt",
    ])
    parser = ArgumentParser(description=descr, epilog=examples, formatter_class=RawTextHelpFormatter, add_help=False)
    parser.add_argument("path", type=valid_file, help="path to portable executable")
    opt = parser.add_argument_group("optional arguments")
    opt.add_argument("-d", "--db", default=DB, type=valid_file,
                     help="path to the custom database of signatures (default: None ; use the embedded DB)")
    opt.add_argument("-e", "--ep-only", action="store_false",
                     help="consider only entry point signatures (default: True)")
    extra = parser.add_argument_group("extra arguments")
    extra.add_argument("-b", "--benchmark", action="store_true",
                       help="enable benchmarking, output in seconds (default: False)")
    extra.add_argument("-h", "--help", action="help", help="show this help message and exit")
    extra.add_argument("-v", "--verbose", action="store_true", help="display debug information (default: False)")
    args = parser.parse_args()
    logging.basicConfig()
    args.logger = logging.getLogger("peid")
    args.logger.setLevel([logging.INFO, logging.DEBUG][args.verbose])
    code = 0
    # execute the tool
    if args.benchmark:
        t1 = perf_counter()
    try:
        r = identify_packer(args.path, args.db, args.ep_only, args.logger)
        dt = str(perf_counter() - t1) if args.benchmark else ""
        if dt != "":
            r.append(dt)
        if len(r) > 0:
            print("\n".join(r))
    except Exception as e:
        args.logger.exception(e)
        code = 1
    return code

